#include "rszvl.h"
#include <cvirte.h>     
#include <userint.h>
#include <visa.h>
#include "RSZVL_Read_multiple_traces_example.h"


/* Useful macros */
#define CHECKERR(fCal) \
    if (status = checkError((fCal)), status < VI_SUCCESS) \
        return status; else

#define ARRAY_LEN 5000

/* Global variables */
static   ViInt32    panelHandle  = 0,
                    traceCount   =1,
                    channelCount =1,
                    traceMeas,
                    traceChannel,
                    traceDisplay;
static   ViInt16    sweepNumber;
volatile ViBoolean  enableTrace  = VI_FALSE;
static   ViSession  instrSession = VI_NULL;

         ViReal64   amplitude [ARRAY_LEN],real[ARRAY_LEN/2],imag[ARRAY_LEN/2];
         ViChar     traceName[21];
         
    
    


/* Utility functions declarations */
ViStatus checkError(ViStatus status);
void    cleanup (ViBoolean deviceClear);
void    EnableButtons(ViBoolean enable);
int     getColor(int count);



int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)
        return -1;  /* out of memory */
    if ((panelHandle = LoadPanel (0, "RSZVL_Read_multiple_traces_example.uir", PANEL)) < 0)
        return -1;
    
    DisplayPanel (panelHandle);
    RunUserInterface ();
    DiscardPanel (panelHandle);
    
    return 0;
}

/*===========================================================================*/
/* Function: Exit Button Callback
/* Purpose:  Closes program window
/*===========================================================================*/
int  CVICALLBACK exitButton(int panel, int control, int event, 
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_COMMIT:
            enableTrace = VI_FALSE;
            if (instrSession != 0)
            {
                rszvl_close (instrSession);
                instrSession = 0;
            }
            QuitUserInterface (0);
            
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Start Button Callback
/* Purpose:  Configures the instrument according to selected measurement
/*           parameters and enables the measurement loop
/*===========================================================================*/
int  CVICALLBACK startButton(int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViChar      instr_desc [256] = "",
                tempString [32]  = "",
                catalogString[256] = "",
                channelString[6];
    ViInt32     id               = 0;
    ViInt32     reset            = VI_FALSE;
    ViInt32     display          = VI_FALSE;
    ViStatus    status           = VI_SUCCESS;
    int         i;
    
    switch (event)
    {
        case EVENT_COMMIT:     /* start button pressed */
            {
            
                /* block mouse events */
                SetWaitCursor (1);
                
                /* read config from panel*/
                GetCtrlVal (panelHandle, PANEL_INSTR_DESC, instr_desc);
                GetCtrlVal (panelHandle, PANEL_IDQUERY, &id);
                GetCtrlVal (panelHandle, PANEL_RESET, &reset);
                GetCtrlVal (panelHandle, PANEL_DISPLAY, &display);
                GetCtrlVal (panelHandle, PANEL_SWEEP_NUMBER, &sweepNumber);
                

                /* disable buttons for configuration */
                EnableButtons(VI_FALSE);
                
                /* initialize driver */
                if (instrSession == VI_NULL )
                    CHECKERR (rszvl_init (instr_desc, id, reset, &instrSession ));
                else if (reset == VI_TRUE ) /* reset device box checked */
                    CHECKERR (rszvl_reset (instrSession));
				
				/* delete default trace (after reset) */
				if (reset == VI_TRUE)
				{
					CHECKERR( rszvl_TraceDelete (instrSession, 1, "Trc1"));
				}
                
                /* reduce number of sweep points to increase speed of configuration*/
                CHECKERR(rszvl_ConfigureSweepPoints(instrSession,1,2));
								
                /* switch device display on/off */
                CHECKERR(rszvl_ConfigureDisplayUpdate (instrSession, display, VI_FALSE));
 
                /* Set instrument to single sweep mode - continous mode is done
                 * programmatically because this is the only way how to
                 * synchronize measurement using *OPC
                 */
                CHECKERR (rszvl_ConfigureAcquisition (instrSession, VI_FALSE, 1));
				
                /*Set parameters for all traces */
                for(i=1;i<=traceCount;i++)
                {
                    /* read trace parameters from table */
                    GetTableCellVal (panelHandle, PANEL_TABLE, MakePoint (1,i), traceName);
                    GetTableCellVal (panelHandle, PANEL_TABLE, MakePoint (2,i), tempString);
                    CHECKERR (GetTableCellRingIndexFromValue (panelHandle, PANEL_TABLE, 0,
                                                                MakePoint (2,i), &traceMeas,
                                                                tempString));
                    GetTableCellVal (panelHandle, PANEL_TABLE, MakePoint (3,i), tempString);
                    CHECKERR (GetTableCellRingIndexFromValue (panelHandle, PANEL_TABLE, 0,
                                                                MakePoint (3,i), &traceChannel,
                                                                tempString));
                    traceChannel++;
                    GetTableCellVal (panelHandle, PANEL_TABLE, MakePoint (4,i), tempString);
                    CHECKERR (GetTableCellRingIndexFromValue (panelHandle, PANEL_TABLE, 0,
                                                                MakePoint (4,i), &traceDisplay,
                                                                tempString));

                    /*Add trace channel (configure channel name, renaming possible)*/
                    sprintf(channelString, "Ch%d", traceChannel);
                    CHECKERR (rszvl_ChannelAdd (instrSession,traceChannel,channelString));
                        
                    /* Create and show this trace?
                     * If the trace has to be shown in diagram area, it is
                     * created here with default trace settings
                     */
                    if(traceDisplay!=VI_NULL)
                    {          
                        /*List all existing traces*/   
                        CHECKERR (rszvl_TraceList (instrSession,traceChannel,256,catalogString));
                        /*Create noexisting trace*/
                        if(NULL == strstr (catalogString, traceName)){
                            CHECKERR (rszvl_TraceAddDiagramArea (instrSession,1,i,traceChannel,traceName));
                        }                        
                    }
                    
                    /* Set trace measurement parameter.
                     * If the trace has not been created yet, it is created here,
                     * but is not shown in diagram area. Otherwise this only assigns
                     * measurement parameter
                     */
                    switch(traceMeas)
                        {
                            case 0:   //S11
                                CHECKERR(rszvl_ConfigureSParameters( instrSession, traceChannel,traceName,1,1));
                                break;
                            case 1:   //S12
                                CHECKERR(rszvl_ConfigureSParameters( instrSession, traceChannel,traceName,1,2));
                                break;
                            case 2:   //S21
                                CHECKERR(rszvl_ConfigureSParameters( instrSession, traceChannel,traceName,2,1));
                                break;
                            case 3:   //S22
                                CHECKERR(rszvl_ConfigureSParameters( instrSession, traceChannel,traceName,2,2));
                                break;
                            case 4:   //Z-S11
                                CHECKERR(rszvl_ConfigureImpedances( instrSession, traceChannel,traceName,1,1));
                                break;
                            case 5:   //Z-S12
                                CHECKERR(rszvl_ConfigureImpedances( instrSession, traceChannel,traceName,1,2));
                                break;
                            case 6:   //Z-S21
                                CHECKERR(rszvl_ConfigureImpedances( instrSession, traceChannel,traceName,2,1));
                                break;
                            case 7:   //Z-S22
                                CHECKERR(rszvl_ConfigureImpedances( instrSession, traceChannel,traceName,2,2));
                                break;
                            case 8:   //Y-S11
                                CHECKERR(rszvl_ConfigureAdmitances( instrSession, traceChannel,traceName,1,1));
                                break;
                            case 9:   //Y-S12
                                CHECKERR(rszvl_ConfigureAdmitances( instrSession, traceChannel,traceName,1,2));
                                break;
                            case 10:  //Y-S21
                                CHECKERR(rszvl_ConfigureAdmitances( instrSession, traceChannel,traceName,2,1));
                                break;
                            case 11:  //Y-S22
                                CHECKERR(rszvl_ConfigureAdmitances( instrSession, traceChannel,traceName,2,2));
                                break;
                        }
                    
                }
                
                /*set number of sweeps for all channels*/
                for(i=1;i<=channelCount;i++)
                {
                    CHECKERR (rszvl_ConfigureSweepPoints(instrSession,i,sweepNumber));
                }
                
                /* start measurements */
                enableTrace = VI_TRUE;
                
                /* enable mouse events*/
                SetWaitCursor (0);
            }
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Stop Button Callback
/* Purpose:  Stops the continuous measurement, in other cases has no sense
/*===========================================================================*/
int CVICALLBACK stopButton (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_COMMIT:
            
            /* stop measurements */
            enableTrace = VI_FALSE;
            
            /* enable buttons for configuration */
            EnableButtons(VI_TRUE);
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Timer Callback
/* Purpose:  Every timer tick the traces are read out of the device and plotted
/*           as a graph. Measurements has to be enabled otherwise callback
/*           returns immediatelly
/*===========================================================================*/
int  CVICALLBACK timer(int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViStatus    temp_status  = VI_SUCCESS,
                status       = VI_SUCCESS;
    ViChar      tempString [10]  = "";
    ViInt32     numberOfValues = 0;
    int         i,j;
    
    switch (event)
    {
        case EVENT_TIMER_TICK:  /* every 100ms */
            
            /* already configured and ready for measurements?*/
            if(enableTrace==VI_FALSE)
                return 0;
            
            /* exclusive execution from now on! */          
            enableTrace = VI_FALSE;
            
            /* initiate measurement */ 
            CHECKERR (rszvl_SANInitiate (instrSession, 15000));
                        
            DeleteGraphPlot (panelHandle, PANEL_GRAPH, -1, VAL_DELAYED_DRAW);

            /* Display traces */
            for(i=1;i<=traceCount;i++)
                {
                    /* Read, if this trace has to be displayed */
                    GetTableCellVal (panelHandle, PANEL_TABLE, MakePoint (4,i), tempString);
                    GetTableCellRingIndexFromValue (panelHandle, PANEL_TABLE, 0,
                                                                MakePoint (4,i), &traceDisplay,
                                                                tempString);
                    
                    /* Display only traces, that have to be displayed */
                    if(traceDisplay==VI_NULL)
                    {
                        continue;
                    }
                    else
                    {
                        /* read trace parameters from table */
                        GetTableCellVal (panelHandle, PANEL_TABLE, MakePoint (1,i), traceName);
                        GetTableCellVal (panelHandle, PANEL_TABLE, MakePoint (2,i), tempString);
                        GetTableCellRingIndexFromValue (panelHandle, PANEL_TABLE, 0,
                                                        MakePoint (2,i), &traceMeas,
                                                        tempString);
                        GetTableCellVal (panelHandle, PANEL_TABLE, MakePoint (3,i), tempString);
                        GetTableCellRingIndexFromValue (panelHandle, PANEL_TABLE, 0,
                                                        MakePoint (3,i), &traceChannel,
                                                        tempString);
                        traceChannel++;
                    
                        /*activate this trace*/
                        CHECKERR (rszvl_TraceSelect (instrSession, traceChannel, traceName));
                        
                        /*read single sweep data*/
                        CHECKERR (rszvl_GetTraceResponseSingleSweepData (instrSession, traceChannel, 1,
                                                                         &numberOfValues, amplitude));
                        /*split real and imag part for plotting*/
                        for(j=0;j<ARRAY_LEN;j+=2)
                        {
                            real[j/2]=amplitude[j];
                            imag[j/2]=amplitude[j+1];
                        }
                        
                        /* Plots a waveform onto a graph control */
                        temp_status = PlotWaveform (panelHandle, PANEL_GRAPH, real,
                                                    numberOfValues/2, VAL_DOUBLE, 1.0, 0.0, 0.0,
                                                    1, VAL_THIN_LINE, VAL_SIMPLE_DOT,
                                                    VAL_SOLID, 1, getColor(i));
                        
                        if( temp_status < 0 )
                        {
                            /* prompt CVI error message */
                            MessagePopup ("Error", GetUILErrorString(temp_status));
                
                            /* cleanup zvl driver session */
                            cleanup(VI_FALSE);
                
                            /* do not free resource */
                            enableTrace = VI_FALSE;
                            return -1;
                        }
                        
                        
                        /* Plots a waveform onto a graph control */
                        temp_status = PlotWaveform (panelHandle, PANEL_GRAPH, imag,
                                                    numberOfValues/2, VAL_DOUBLE, 1.0, 0.0, 0.0,
                                                    1, VAL_THIN_LINE, VAL_SIMPLE_DOT,
                                                    VAL_SOLID, 1, getColor(i+10));
                        if( temp_status < 0 )
                        {
                            /* prompt CVI error message */
                            MessagePopup ("Error", GetUILErrorString(temp_status));
                
                            /* cleanup zvl driver session */
                            cleanup(VI_FALSE);
                
                            /* do not free resource */
                            enableTrace = VI_FALSE;
                            return -1;
                        }
                        
                    }
                }
        
            /* free resource */         
            enableTrace = VI_TRUE;
            
            break;
    }
    return 0;
}


/*===========================================================================*/
/* Function: checkError
/* Purpose:  If passed status indicates an failure, the error message will
/*           be displayed.
/*===========================================================================*/
ViStatus checkError (ViStatus status)
{
    ViChar  error_message [256];
    ViChar  error_buffer [1024];
    ViChar*  p2buf;
    ViInt32 error;
    
    if (status < VI_SUCCESS)
    {
        /* Converts a status code returned by an instrument driver function into a user-readable string */
        rszvl_error_message (instrSession, status, error_message);
        p2buf = error_buffer + sprintf (error_buffer, "Primary Error: 0x%08X, %s\n", status, error_message);
        
        /* This function reads an error code and a message from the instrument's error queue */
        rszvl_error_query (instrSession, &error, error_message);
        
        SetWaitCursor (0);
        if (error != VI_SUCCESS)
        {
            sprintf (p2buf, "Secondary Error: 0x%08X, %s\n", error, error_message);
        }

        MessagePopup ("Error", error_buffer);
        
        /* close session */
        cleanup(VI_TRUE);
        
        /* enable buttons for configuration */
        EnableButtons(VI_TRUE);
    }
    return status;
}


/*===========================================================================*/
/* Function: Cleanup
/* Purpose:  Closes the VISA session to the instrument and in case of error
/*           queries the error and shows the dialog box with the passed error
/*           message. In case Device Clear is requested, it is necessary to 
/*           set the argument deviceClear to VI_TRUE.
/*===========================================================================*/
void cleanup (ViBoolean deviceClear)
{

    /* Device Clear is a low-level command and should be used in case the
     * instrument is waiting for Operation Complete to cancel the wait.
     * It is useful for instance in case of incorrect external trigger
     * when the instrument does not respond to any other command because of
     * waiting for trigger.
     */
    if (deviceClear)
    {
        viClear (instrSession);
        viPrintf (instrSession, "*CLS\n");
    }
    
    /* close instrument session */
    rszvl_close (instrSession);
    instrSession = VI_NULL;
    
    /* no measurements */
    enableTrace  = VI_FALSE;
    
    /* enable GUI */
    EnableButtons (1);
}


/*===========================================================================*/
/* Function: EnableButtons
/* Purpose:  This function dis-/enables GUI while measuring/configuration.
/*===========================================================================*/
void EnableButtons(ViBoolean enable){

    SetCtrlAttribute (panelHandle, PANEL_NEW_CHANNEL,ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_NEW_TRACE,  ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_START,      ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_RESET,      ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_IDQUERY,    ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_INSTR_DESC, ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_DISPLAY,    ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_SWEEP_NUMBER,ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_TABLE,      ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_STOP,       ATTR_DIMMED,  enable);
    
}


/*===========================================================================*/
/* Function: createNewTrace
/* Purpose:  This function creates new trace a adds it into table.
/*===========================================================================*/
int  CVICALLBACK createNewTrace(int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    int     i=1;
    char    traceNameString[9],
            channelString[6];
    
    switch (event)
    {
        case EVENT_COMMIT:
            
            /*make new row*/
            InsertTableRows (panelHandle, PANEL_TABLE, traceCount+1, 1,
                             VAL_USE_MASTER_CELL_TYPE);
            
            /*insert trace name*/
            sprintf (traceNameString, "Trc%d", traceCount+1);
            SetTableCellVal (panelHandle, PANEL_TABLE, MakePoint(1, traceCount+1),
                             traceNameString);
            
            /*insert values to Ring controls*/
            InsertTableCellRingItem (panelHandle, PANEL_TABLE, MakePoint(2, traceCount+1), -1, "S11");
            InsertTableCellRingItem (panelHandle, PANEL_TABLE, MakePoint(2, traceCount+1), -1, "S12");
            InsertTableCellRingItem (panelHandle, PANEL_TABLE, MakePoint(2, traceCount+1), -1, "S21");
            InsertTableCellRingItem (panelHandle, PANEL_TABLE, MakePoint(2, traceCount+1), -1, "S22");
            InsertTableCellRingItem (panelHandle, PANEL_TABLE, MakePoint(2, traceCount+1), -1, "Z-S11");
            InsertTableCellRingItem (panelHandle, PANEL_TABLE, MakePoint(2, traceCount+1), -1, "Z-S12");
            InsertTableCellRingItem (panelHandle, PANEL_TABLE, MakePoint(2, traceCount+1), -1, "Z-S21");
            InsertTableCellRingItem (panelHandle, PANEL_TABLE, MakePoint(2, traceCount+1), -1, "Z-S22");
            InsertTableCellRingItem (panelHandle, PANEL_TABLE, MakePoint(2, traceCount+1), -1, "Y-S11");
            InsertTableCellRingItem (panelHandle, PANEL_TABLE, MakePoint(2, traceCount+1), -1, "Y-S12");
            InsertTableCellRingItem (panelHandle, PANEL_TABLE, MakePoint(2, traceCount+1), -1, "Y-S21");
            InsertTableCellRingItem (panelHandle, PANEL_TABLE, MakePoint(2, traceCount+1), -1, "Y-S22");
            
            for (i; i<=channelCount; i++)
            {
                sprintf(channelString, "Ch%d", i);
                InsertTableCellRingItem (panelHandle, PANEL_TABLE, MakePoint(3, traceCount+1), -1, channelString);
            }
            
            InsertTableCellRingItem (panelHandle, PANEL_TABLE, MakePoint(4, traceCount+1), -1, "Off");
            InsertTableCellRingItem (panelHandle, PANEL_TABLE, MakePoint(4, traceCount+1), -1, "On");
            
            
            traceCount++;
            
            break;
    }
    return 0;
}


/*===========================================================================*/
/* Function: createNewChannel
/* Purpose:  This function creates new channel a adds it into table ring control.
/*===========================================================================*/
int  CVICALLBACK createNewChannel(int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    int     i=1;
    char    channelString[6];
    
    switch (event)
    {
        case EVENT_COMMIT:
            
            
            sprintf(channelString, "Ch%d", channelCount+1); 
                            
            for (i; i<=traceCount; i++)
            {
                InsertTableCellRingItem (panelHandle, PANEL_TABLE, MakePoint(3, i), -1, channelString);
            }
            
            
            channelCount++;
            
            break;
    }
    return 0;
}



/*===========================================================================*/
/* Function: getColor
/* Purpose:  This function returns color for graph plot.
/*===========================================================================*/
int getColor(int count)
{
    
    while (count>12)
    {
        count-=12;
    }
    
    switch (count)
    {
        case 1:
            return VAL_RED;
        case 2:
            return VAL_GREEN;
        case 3:
            return VAL_BLUE;
        case 4:
            return VAL_CYAN;
        case 5:
            return VAL_MAGENTA;
        case 6:
            return VAL_YELLOW;
        case 7:
            return VAL_DK_BLUE;
        case 8:
            return VAL_DK_CYAN;
        case 9:
            return VAL_DK_GREEN;
        case 10:
            return VAL_DK_MAGENTA;
        case 11:
            return VAL_DK_RED;
        case 12:
            return VAL_DK_YELLOW;
    }
    return 0;   
}
